#!/usr/bin/python


# python eigenvalues.py <entry>


from scipy.io import mmread
import scipy.sparse as sp
import numpy as np
import itertools
from numpy import linalg as la
import os

from matplotlib import pyplot as plt
from matplotlib.animation import FuncAnimation


from mpmath import *
mp.dps = 40; mp.pretty = False



import configparser
import ast



import sys

commline = sys.argv
print ('arguments:',commline)

job = commline[1]


################################################### INI file parsing
Config = configparser.ConfigParser()
Config.read(os.path.splitext(commline[0])[0]+".ini")
print ('ini:',Config.sections())



def ConfigSectionMap(section):
    dict1 = {}
    options = Config.options(section)
    for option in options:
        try:
            dict1[option] = Config.get(section, option)
            if dict1[option] == -1:
                DebugPrint("skip: %s" % option)
        except:
            print("exception on %s!" % option)
            dict1[option] = None
    return dict1



prefix = ConfigSectionMap("common")['prefix']
suffix = ConfigSectionMap("common")['suffix']

pomo = ConfigSectionMap(job)['masses'].split(",")
masses = [float(i) for i in pomo]

pomo = ConfigSectionMap(job)['stiff']
stiff = float(pomo)

pomo = ConfigSectionMap(job)['which']
which = int(pomo)

filename = ConfigSectionMap(job)['filename']








######################################### calculation of eigenvalues


print ("\n ----- mp\n")


# full matrix
A = mmread(prefix+filename+suffix)
Fall = A.todense()
dimall = np.shape(Fall)[0]
print ('n =',dimall)

uall = mp.matrix(dimall)
for i in range(dimall):
   for j in range(dimall):
      Fall[i,j] /= sqrt(masses[i]*masses[j])
      uall[i,j]=Fall[i,j]


print ('|F|=',mp.det(uall))
vlall = mp.eig(uall)


#print vlall[0]
#print vlall[1]


ktisku = np.zeros(dimall,dtype=complex)
for i in range(dimall):
  ktisku[i]=complex(sqrt(vlall[0][i]*stiff))

print (ktisku)
np.savetxt(prefix+filename+"_frequencies.txt",np.sort(ktisku),fmt="%.5f")












########################################################## plotting


eigenvalue = abs(ktisku[which])
eigenvector = np.array(vlall[1][:,which],dtype=float)

print ('\n',eigenvalue)
print (eigenvector)



# plotting

fig = plt.figure()
ax = plt.axes(xlim=(-2.0, 2.0), ylim=(-1, 1))


mat, = ax.plot([], [], 'o')

def init():
    mat.set_data([], [])
    return mat,



def animate(i):
    y = np.zeros(dimall)
    x0 = np.array(np.linspace(-1.0,1.0,num=dimall))
    x = x0 + 0.2*eigenvector * sin(eigenvalue * 0.1 * i)
    mat.set_data(x, y)
    return mat,




anim = FuncAnimation(fig, animate, init_func=init,frames=200, interval=20, blit=False)


plt.show()





